==> standwithukraine-1.2.4.tar.gz: standwithukraine/src/Form/StandWithUkraineForm.php <==
<?php

namespace Drupal\standwithukraine\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\Element;
use Drupal\Core\Url;
use Drupal\standwithukraine\Service\StandWithUkraineFaviconInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure StandWithUkraine settings for this site.
 */
class StandWithUkraineForm extends ConfigFormBase {

  /**
   * The suggested websites list.
   */
  const URLS = [
    'https://supportukrainenow.org',
    'https://war.ukraine.ua/',
  ];

  /**
   * The cache.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The favicon.
   *
   * @var \Drupal\standwithukraine\Service\StandWithUkraineFaviconInterface
   */
  protected $favicon;

  /**
   * The time.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);

    $instance->cache = $container->get('cache.default');
    $instance->favicon = $container->get('standwithukraine.favicon');
    $instance->time = $container->get('datetime.time');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['standwithukraine.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'standwithukraine_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config($this->getEditableConfigNames()[0]);

    $form['url'] = [
      '#type' => 'url',
      '#title' => $this->t('Link'),
      '#description' => sprintf(
        '%s %s.',
        $this->t('For example:'),
        implode(
          ', ',
          array_map(
            function ($url) {
              return Link::fromTextAndUrl(
                parse_url($url, PHP_URL_HOST),
                Url::fromUri($url, ['target' => '_blank'])
              )->toString();
            },
            self::URLS
          )
        )
      ),
      '#default_value' => $config->get('url'),
    ];

    $form['position'] = [
      '#type' => 'radios',
      '#title' => $this->t('Position'),
      '#description' => $this->t('Select on which side of the page a message
should be displayed.'),
      '#options' => [
        'left' => $this->t('Left'),
        '' => $this->t('Top (default)'),
        'right' => $this->t('Right'),
      ],
      '#default_value' => $config->get('position') ?? '',
    ];

    $form['offset'] = [
      '#type' => 'number',
      '#title' => $this->t('Offset'),
      '#description' => $this->t('How many pixels down the message should be
shifted down the page. When a field is empty then the message will be shifted
down by 20 percent from the page height. Available only for the left or right
position of the message.'),
      '#min' => 0,
      '#field_suffix' => $this->t('pixels'),
      '#default_value' => $config->get('offset'),
      '#states' => [
        'enabled' => [
          ':input[name="position"]' => [
            ['value' => 'left'],
            ['value' => 'right'],
          ],
        ],
      ],
    ];

    $form['favicon'] = [
      '#type' => 'details',
      '#title' => 'Favicon',
      '#tree' => TRUE,
      '#open' => FALSE,
    ];

    $form['favicon']['enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable'),
      '#default_value' => !$config->get('favicon.disable'),
    ];

    $form['favicon']['ratios'] = [
      '#type' => 'details',
      '#title' => $this->t('Ratios'),
      '#description' => $this->t('Define how many percentages of icon height
should be dedicated to blue color.'),
      '#open' => FALSE,
      '#states' => [
        'disabled' => [
          ':input[name="favicon[enable]"]' => [
            'checked' => FALSE,
          ],
        ],
      ],
    ];

    $theme_config = $this->config('system.theme');
    $ratios = (array) $config->get('favicon.ratios');
    $suffix = '?timestamp=' . $this->time->getRequestTime();
    $count = 0;

    $theme_types = [
      'default' => $this->t('Default theme'),
      'admin' => $this->t('Admin theme'),
    ];

    foreach ($theme_types as $theme_type => $title) {
      $theme_name = $theme_config->get($theme_type);

      if (empty($theme_name)) {
        continue;
      }

      $favicon = $this->favicon->build($theme_name);

      if (!isset($favicon[$theme_name])) {
        continue;
      }

      $count++;

      if (isset($ratios[$theme_name])) {
        $ratio = (int) $ratios[$theme_name];
      }
      elseif (in_array($theme_name, StandWithUkraineFaviconInterface::THEMES)) {
        $ratio = StandWithUkraineFaviconInterface::DEFINED_RATIO;
      }
      else {
        $ratio = StandWithUkraineFaviconInterface::UNDEFINED_RATIO;
      }

      $custom = $ratio !== StandWithUkraineFaviconInterface::UNDEFINED_RATIO;

      $form['favicon']['#open'] |= $custom;
      $form['favicon']['ratios']['#open'] |= $custom;

      $form['favicon']['ratios'][$theme_name] = [
        '#type' => 'details',
        '#title' => $title,
        '#open' => $custom,
      ];

      $form['favicon']['ratios'][$theme_name]['preview'] = [
        '#theme' => 'table',
        '#caption' => $this->t('Preview'),
        '#attributes' => [
          'style' => 'width: auto;',
        ],
      ];

      $rows = &$form['favicon']['ratios'][$theme_name]['preview']['#rows'];

      foreach ($favicon[$theme_name] as $dimensions => $url) {
        $rows['images'][] = [
          'data' => [
            '#type' => 'html_tag',
            '#tag' => 'img',
            '#attributes' => [
              'src' => $url . $suffix,
            ],
          ],
          'style' => 'text-align: center; vertical-align: bottom;',
        ];

        $rows['dimensions'][] = [
          'data' => $dimensions,
          'style' => 'text-align: center; vertical-align: bottom;',
        ];
      }

      if (($columns = count($rows['images'])) > 1) {
        for ($current_column = 0; $current_column < $columns - 1; $current_column++) {
          for ($next_column = $current_column + 1; $next_column < $columns; $next_column++) {
            $sizes = array_map(
              function ($column) use ($rows) {
                return explode('x', $rows['dimensions'][$column]['data'])[0];
              },
              [$current_column, $next_column],
            );

            if ($sizes[0] > $sizes[1]) {
              foreach (array_keys($rows) as $row) {
                $buffer = $rows[$row][$current_column];
                $rows[$row][$current_column] = $rows[$row][$next_column];
                $rows[$row][$next_column] = $buffer;
              }
            }
          }
        }
      }

      $form['favicon']['ratios'][$theme_name]['ratio'] = [
        '#type' => 'number',
        '#min' => 10,
        '#max' => 90,
        '#default_value' => $ratio,
        '#field_prefix' => $this->t('Ratio'),
        '#field_suffix' => '%',
      ];
    }

    $form['favicon']['#open'] &= $form['favicon']['enable']['#default_value'];
    $form['favicon']['#access'] = $count > 0;

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config($this->getEditableConfigNames()[0]);

    foreach (['url', 'position', 'offset'] as $key) {
      $config->set($key, $form_state->getValue($key));
    }

    $config->set(
      'favicon.disable',
      !$form_state->getValue(['favicon', 'enable'])
    );

    foreach (Element::children($form['favicon']['ratios']) as $theme_name) {
      $config->set(
        'favicon.ratios.' . $theme_name,
        $form_state->getValue(['favicon', 'ratios', $theme_name, 'ratio'])
      );
    }

    $config->save();

    $this->cache->invalidate('standwithukraine');

    parent::submitForm($form, $form_state);
  }

}

==> standwithukraine-1.2.5.tar.gz: standwithukraine/src/Form/StandWithUkraineForm.php <==
<?php

namespace Drupal\standwithukraine\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\Element\CompositeFormElementTrait;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\Url;
use Drupal\standwithukraine\Service\StandWithUkraineFaviconInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure StandWithUkraine settings for this site.
 */
class StandWithUkraineForm extends ConfigFormBase implements TrustedCallbackInterface {

  use CompositeFormElementTrait;

  /**
   * The suggested websites list.
   */
  const URLS = [
    'https://supportukrainenow.org',
    'https://war.ukraine.ua/',
  ];

  const TYPES = [FALSE => 'foreground', TRUE => 'background'];

  /**
   * The cache.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The favicon.
   *
   * @var \Drupal\standwithukraine\Service\StandWithUkraineFaviconInterface
   */
  protected $favicon;

  /**
   * The time.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);

    $instance->cache = $container->get('cache.default');
    $instance->favicon = $container->get('standwithukraine.favicon');
    $instance->time = $container->get('datetime.time');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['standwithukraine.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'standwithukraine_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config($this->getEditableConfigNames()[0]);

    $form['url'] = [
      '#type' => 'url',
      '#title' => $this->t('Link'),
      '#description' => sprintf(
        '%s %s.',
        $this->t('For example:'),
        implode(
          ', ',
          array_map(
            function ($url) {
              return Link::fromTextAndUrl(
                parse_url($url, PHP_URL_HOST),
                Url::fromUri($url, ['target' => '_blank'])
              )->toString();
            },
            self::URLS
          )
        )
      ),
      '#default_value' => $config->get('url'),
    ];

    $form['position'] = [
      '#type' => 'radios',
      '#title' => $this->t('Position'),
      '#description' => $this->t('Select on which side of the page a message
should be displayed.'),
      '#options' => [
        'left' => $this->t('Left'),
        '' => $this->t('Top (default)'),
        'right' => $this->t('Right'),
      ],
      '#default_value' => $config->get('position') ?? '',
    ];

    $form['offset'] = [
      '#type' => 'number',
      '#title' => $this->t('Offset'),
      '#description' => $this->t('How many pixels down the message should be
shifted down the page. When a field is empty then the message will be shifted
down by 20 percent from the page height. Available only for the left or right
position of the message.'),
      '#min' => 0,
      '#field_suffix' => $this->t('pixels'),
      '#default_value' => $config->get('offset'),
      '#states' => [
        'enabled' => [
          ':input[name="position"]' => [
            ['value' => 'left'],
            ['value' => 'right'],
          ],
        ],
      ],
    ];

    $form['favicon'] = [
      '#type' => 'details',
      '#title' => 'Favicon',
      '#tree' => TRUE,
      '#open' => FALSE,
    ];

    $form['favicon']['enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable'),
      '#description' => $this->t('Activate functionality for styling favicon.'),
      '#default_value' => !$config->get('favicon.disable'),
    ];

    $form['favicon']['themes'] = [
      '#type' => 'details',
      '#title' => $this->t('Themes'),
      '#description' => $this->t('Configure favicon for default theme or admin
theme.'),
      '#open' => FALSE,
      '#states' => [
        'disabled' => [
          ':input[name="favicon[enable]"]' => [
            'checked' => FALSE,
          ],
        ],
      ],
    ];

    $theme_config = $this->config('system.theme');
    $themes = (array) $config->get('favicon.themes');
    $suffix = '?timestamp=' . $this->time->getRequestTime();
    $count = 0;

    /** @var \Drupal\Core\StringTranslation\TranslatableMarkup $title */
    foreach ([$this->t('Default'), $this->t('Admin')] as $title) {
      $theme_type = strtolower($title->getUntranslatedString());
      $theme_name = $theme_config->get($theme_type);

      if (empty($theme_name)) {
        continue;
      }

      $favicon = $this->favicon->build($theme_name);

      if (!isset($favicon[$theme_name])) {
        continue;
      }

      $count++;

      if (isset($themes[$theme_name])) {
        $ratio = (int) $themes[$theme_name]['ratio'];
      }
      elseif (in_array($theme_name, StandWithUkraineFaviconInterface::THEMES)) {
        $ratio = StandWithUkraineFaviconInterface::DEFINED_RATIO;
      }
      else {
        $ratio = StandWithUkraineFaviconInterface::UNDEFINED_RATIO;
      }

      $custom = $ratio !== StandWithUkraineFaviconInterface::UNDEFINED_RATIO;

      $form['favicon']['#open'] |= $custom;
      $form['favicon']['themes']['#open'] |= $custom;

      $form['favicon']['themes'][$theme_name] = [
        '#type' => 'details',
        '#title' => $title,
        '#open' => $custom,
      ];

      $form['favicon']['themes'][$theme_name]['preview'] = [
        '#theme' => 'table',
        '#caption' => $this->t('Preview'),
        '#attributes' => [
          'class' => ['preview'],
        ],
        '#attached' => [
          'library' => ['standwithukraine/form'],
        ],
      ];

      $rows = &$form['favicon']['themes'][$theme_name]['preview']['#rows'];

      foreach ($favicon[$theme_name] as $dimensions => $url) {
        $rows['images'][]['data'] = [
          '#type' => 'html_tag',
          '#tag' => 'img',
          '#attributes' => [
            'src' => $url . $suffix,
          ],
        ];

        $rows['dimensions'][]['data'] = $dimensions;
      }

      if (($columns = count($rows['images'])) > 1) {
        for ($current_column = 0; $current_column < $columns - 1; $current_column++) {
          for ($next_column = $current_column + 1; $next_column < $columns; $next_column++) {
            $sizes = array_map(
              function ($column) use ($rows) {
                return explode('x', $rows['dimensions'][$column]['data'])[0];
              },
              [$current_column, $next_column],
            );

            if ($sizes[0] > $sizes[1]) {
              foreach (array_keys($rows) as $row) {
                $buffer = $rows[$row][$current_column];
                $rows[$row][$current_column] = $rows[$row][$next_column];
                $rows[$row][$next_column] = $buffer;
              }
            }
          }
        }
      }

      $form['favicon']['themes'][$theme_name]['ratio'] = [
        '#type' => 'number',
        '#title' => $this->t('Ratio'),
        '#description' => $this->t('Define how many percentages of icon height
should be dedicated to blue color.'),
        '#min' => 10,
        '#max' => 90,
        '#default_value' => $ratio,
        '#field_suffix' => '%',
      ];

      $form['favicon']['themes'][$theme_name]['type'] = [
        '#type' => 'radios',
        '#title' => $this->t('Type'),
        '#options' => [
          'foreground' => $this->t('Foreground'),
          'background' => $this->t('Background'),
        ],
        '#default_value' => self::TYPES[(bool) ($themes[$theme_name]['background'] ?? 0)],
        '#pre_render' => array_map(function ($callback) {
          return [self::class, $callback];
        }, self::trustedCallbacks()),
      ];
    }

    $form['favicon']['#open'] &= $form['favicon']['enable']['#default_value'];
    $form['favicon']['#access'] = $count > 0;

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config($this->getEditableConfigNames()[0]);

    foreach (['url', 'position', 'offset'] as $key) {
      $config->set($key, $form_state->getValue($key));
    }

    $config->set(
      'favicon.disable',
      !$form_state->getValue(['favicon', 'enable'])
    );

    foreach (Element::children($form['favicon']['themes']) as $theme_name) {
      $path = ['favicon', 'themes', $theme_name];
      $name = implode('.', $path) . '.';

      $config
        ->set($name . 'ratio', $form_state->getValue([...$path, 'ratio']))
        ->set(
          $name . 'background',
          array_search($form_state->getValue([...$path, 'type']), self::TYPES)
        );
    }

    $config->save();

    $this->cache->invalidate('standwithukraine');

    parent::submitForm($form, $form_state);
  }

  /**
   * Describe both types of favicon styling.
   *
   * @param array $element
   *   An associative array containing the properties and children of the
   *   element.
   *
   * @return array
   *   The changed form element render array.
   */
  public static function preRenderType(array $element) {
    $element['foreground']['#description'] = t('Change colors of the visible
part of images and leave the transparent background as is.');

    $element['background']['#description'] = t('Add the Ukraine flag behind of
image visible part and leave the second one as is.');

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['preRenderCompositeFormElement', 'preRenderType'];
  }

}
